searchNodes=[{"doc":"Provides a standard API to handle keyword-list-based options. NimbleOptions allows developers to create schemas using a pre-defined set of options and types. The main benefits are: A single unified way to define simple static options Config validation against schemas Automatic doc generation Schema options These are the options supported in a schema . They are what defines the validation for the items in the given schema. :type - The type of the option item. The default value is :any . :required ( boolean/0 ) - Defines if the option item is required. The default value is false . :default ( term/0 ) - The default value for the option item if that option is not specified. This value is validated according to the given :type . This means that you cannot have, for example, type: :integer and use default: &quot;a string&quot; . :keys ( keyword/0 ) - Available for types :keyword_list , :non_empty_keyword_list , and :map , it defines which set of keys are accepted for the option item. The value of the :keys option is a schema itself. For example: keys: [foo: [type: :atom]] . Use :* as the key to allow multiple arbitrary keys and specify their schema: keys: [*: [type: :integer]] . :deprecated ( String.t/0 ) - Defines a message to indicate that the option item is deprecated. The message will be displayed as a warning when passing the item. :doc - The documentation for the option item. :subsection ( String.t/0 ) - The title of separate subsection of the options' documentation :type_doc - The type doc to use in the documentation for the option item. If false , no type documentation is added to the item. If it's a string, it can be anything. For example, you can use &quot;a list of PIDs&quot; , or you can use a typespec reference that ExDoc can link to the type definition, such as &quot;`t:binary/0`&quot; . You can use Markdown in this documentation. If the :type_doc option is not present, NimbleOptions tries to produce a type documentation automatically if it can do it unambiguously. For example, if type: :integer , NimbleOptions will use integer/0 as the auto-generated type doc. :rename_to ( atom/0 ) - Deprecated in v0.5.0. Types :any - Any type. :keyword_list - A keyword list. :non_empty_keyword_list - A non-empty keyword list. :map - A map consisting of :atom keys. Shorthand for {:map, :atom, :any} . Keys can be specified using the keys option. {:map, key_type, value_type} - A map consisting of key_type keys and value_type values. :atom - An atom. :string - A string. :boolean - A boolean. :integer - An integer. :non_neg_integer - A non-negative integer. :pos_integer - A positive integer. :float - A float. :timeout - A non-negative integer or the atom :infinity . :pid - A PID (process identifier). :reference - A reference (see reference/0 ). :mfa - A named function in the format {module, function, arity} where arity is a list of arguments. For example, {MyModule, :my_fun, [arg1, arg2]} . :mod_arg - A module along with arguments, e.g. {MyModule, [arg1, arg2]} . Usually used for process initialization using start_link and friends. {:fun, arity} - Any function with the specified arity. {:in, choices} - A value that is a member of one of the choices . choices should be a list of terms or a Range . The value is an element in said list of terms, that is, value in choices is true . This was previously called :one_of and the :in name is available since version 0.3.3 ( :one_of has been removed in v0.4.0). {:custom, mod, fun, args} - A custom type. The related value must be validated by mod.fun(values, ...args) . The function should return {:ok, value} or {:error, message} . {:or, subtypes} - A value that matches one of the given subtypes . The value is matched against the subtypes in the order specified in the list of subtypes . If one of the subtypes matches and updates (casts) the given value, the updated value is used. For example: {:or, [:string, :boolean, {:fun, 2}]} . If one of the subtypes is a keyword list or map, you won't be able to pass :keys directly. For this reason, :keyword_list , :non_empty_keyword_list , and :map are special cased and can be used as subtypes with {:keyword_list, keys} , {:non_empty_keyword_list, keys} or {:map, keys} . For example, a type such as {:or, [:boolean, keyword_list: [enabled: [type: :boolean]]]} would match either a boolean or a keyword list with the :enabled boolean option in it. {:list, subtype} - A list where all elements match subtype . subtype can be any of the accepted types listed here. Empty lists are allowed. The resulting validated list contains the validated (and possibly updated) elements, each as returned after validation through subtype . For example, if subtype is a custom validator function that returns an updated value, then that updated value is used in the resulting list. Validation fails at the first element that is invalid according to subtype . If subtype is a keyword list or map, you won't be able to pass :keys directly. For this reason, :keyword_list , :non_empty_keyword_list , and :map are special cased and can be used as the subtype by using {:keyword_list, keys} , {:non_empty_keyword_list, keys} or {:keyword_list, keys} . For example, a type such as {:list, {:keyword_list, enabled: [type: :boolean]}} would a list of keyword lists , where each keyword list in the list could have the :enabled boolean option in it. {:tuple, list_of_subtypes} - A tuple as described by tuple_of_subtypes . list_of_subtypes must be a list with the same length as the expected tuple. Each of the list's elements must be a subtype that should match the given element in that same position. For example, to describe 3-element tuples with an atom, a string, and a list of integers you would use the type {:tuple, [:atom, :string, {:list, :integer}]} . Available since v0.4.1 . Example iex&gt; schema = [ ...&gt; producer : [ ...&gt; type : :non_empty_keyword_list , ...&gt; required : true , ...&gt; keys : [ ...&gt; module : [ required : true , type : :mod_arg ] , ...&gt; concurrency : [ ...&gt; type : :pos_integer , ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; config = [ ...&gt; producer : [ ...&gt; concurrency : 1 , ...&gt; ] ...&gt; ] ...&gt; ...&gt; { :error , % NimbleOptions.ValidationError { } = error } = NimbleOptions . validate ( config , schema ) ...&gt; Exception . message ( error ) &quot;required :module option not found, received options: [:concurrency] (in options [:producer])&quot; Nested option items NimbleOptions allows option items to be nested so you can recursively validate any item down the options tree. Example iex&gt; schema = [ ...&gt; producer : [ ...&gt; required : true , ...&gt; type : :non_empty_keyword_list , ...&gt; keys : [ ...&gt; rate_limiting : [ ...&gt; type : :non_empty_keyword_list , ...&gt; keys : [ ...&gt; interval : [ required : true , type : :pos_integer ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; config = [ ...&gt; producer : [ ...&gt; rate_limiting : [ ...&gt; interval : :oops! ...&gt; ] ...&gt; ] ...&gt; ] ...&gt; ...&gt; { :error , % NimbleOptions.ValidationError { } = error } = NimbleOptions . validate ( config , schema ) ...&gt; Exception . message ( error ) &quot;invalid value for :interval option: expected positive integer, got: :oops! (in options [:producer, :rate_limiting])&quot; Validating Schemas Each time validate/2 is called, the given schema itself will be validated before validating the options. In most applications the schema will never change but validating options will be done repeatedly. To avoid the extra cost of validating the schema, it is possible to validate the schema once, and then use that valid schema directly. This is done by using the new!/1 function first, and then passing the returned schema to validate/2 . Example iex&gt; raw_schema = [ ...&gt; hostname : [ ...&gt; required : true , ...&gt; type : :string ...&gt; ] ...&gt; ] ...&gt; ...&gt; schema = NimbleOptions . new! ( raw_schema ) ...&gt; NimbleOptions . validate ( [ hostname : &quot;elixir-lang.org&quot; ] , schema ) { :ok , hostname : &quot;elixir-lang.org&quot; } Calling new!/1 from a function that receives options will still validate the schema each time that function is called. Declaring the schema as a module attribute is supported: @options_schema NimbleOptions . new! ( [ ... ] ) This schema will be validated at compile time. Calling docs/1 on that schema is also supported.","ref":"NimbleOptions.html","title":"NimbleOptions","type":"module"},{"doc":"Returns documentation for the given schema. You can use this to inject documentation in your docstrings. For example, say you have your schema in a module attribute: @options_schema [ ... ] With this, you can use docs/1 to inject documentation: @doc &quot;Supported options: \\n \#{ NimbleOptions . docs ( @options_schema ) } &quot; Options :nest_level - an integer deciding the &quot;nest level&quot; of the generated docs. This is useful when, for example, you use docs/2 inside the :doc option of another schema. For example, if you have the following nested schema: nested_schema = [ allowed_messages : [ type : :pos_integer , doc : &quot;Allowed messages.&quot; ] , interval : [ type : :pos_integer , doc : &quot;Interval.&quot; ] ] then you can document it inside another schema with its nesting level increased: schema = [ producer : [ type : { :or , [ :string , keyword_list : nested_schema ] } , doc : &quot;Either a string or a keyword list with the following keys: \\n \\n &quot; &lt;&gt; NimbleOptions . docs ( nested_schema , nest_level : 1 ) ] , other_key : [ type : :string ] ]","ref":"NimbleOptions.html#docs/2","title":"NimbleOptions.docs/2","type":"function"},{"doc":"Validates the given schema and returns a wrapped schema to be used with validate/2 . If the given schema is not valid, raises a NimbleOptions.ValidationError .","ref":"NimbleOptions.html#new!/1","title":"NimbleOptions.new!/1","type":"function"},{"doc":"Returns the quoted typespec for any option described by the given schema. The returned quoted code represents the type union for all possible keys in the schema, alongside their type. Nested keyword lists are spec'ed as keyword/0 . Usage Because of how typespecs are treated by the Elixir compiler, you have to use unquote/1 on the return value of this function to use it in a typespec: @type option ( ) :: unquote ( NimbleOptions . option_typespec ( my_schema ) ) This function returns the type union for a single option: to give you flexibility to combine it and use it in your own typespecs. For example, if you only validate part of the options through NimbleOptions, you could write a spec like this: @type my_option ( ) :: { :my_opt1 , integer ( ) } | { :my_opt2 , boolean ( ) } | unquote ( NimbleOptions . option_typespec ( my_schema ) ) If you want to spec a whole schema, you could write something like this: @type options ( ) :: [ unquote ( NimbleOptions . option_typespec ( my_schema ) ) ] Example schema = [ int : [ type : :integer ] , number : [ type : { :or , [ :integer , :float ] } ] ] @type option ( ) :: unquote ( NimbleOptions . option_typespec ( schema ) ) The code above would essentially compile to: @type option ( ) :: { :int , integer ( ) } | { :number , integer ( ) | float ( ) }","ref":"NimbleOptions.html#option_typespec/1","title":"NimbleOptions.option_typespec/1","type":"function"},{"doc":"Validate the given options with the given schema . See the module documentation for what a schema is. If the validation is successful, this function returns {:ok, validated_options} where validated_options is a keyword list. If the validation fails, this function returns {:error, validation_error} where validation_error is a NimbleOptions.ValidationError struct explaining what's wrong with the options. You can use raise/1 with that struct or Exception.message/1 to turn it into a string.","ref":"NimbleOptions.html#validate/2","title":"NimbleOptions.validate/2","type":"function"},{"doc":"Validates the given options with the given schema and raises if they're not valid. This function behaves exactly like validate/2 , but returns the options directly if they're valid or raises a NimbleOptions.ValidationError exception otherwise.","ref":"NimbleOptions.html#validate!/2","title":"NimbleOptions.validate!/2","type":"function"},{"doc":"A schema. See the module documentation for more information.","ref":"NimbleOptions.html#t:schema/0","title":"NimbleOptions.schema/0","type":"type"},{"doc":"The NimbleOptions struct embedding a validated schema. See the Validating Schemas section in the module documentation.","ref":"NimbleOptions.html#t:t/0","title":"NimbleOptions.t/0","type":"type"},{"doc":"An error that is returned (or raised) when options are invalid. Only the following documented fields are considered public. All other fields are considered private and should not be referenced: :key - The key that did not successfully validate. :keys_path - If the key is nested, this is the path to the key. :value - The value that failed to validate. This field is nil if there was no value provided. Since this is an exception, you can either raise it directly with raise/1 or turn it into a message string with Exception.message/1 .","ref":"NimbleOptions.ValidationError.html","title":"NimbleOptions.ValidationError","type":"exception"},{"doc":"","ref":"NimbleOptions.ValidationError.html#t:t/0","title":"NimbleOptions.ValidationError.t/0","type":"type"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Support generating typespecs for :tuple , :map , and {:map, key, value} options","ref":"changelog.html#v0-5-1","title":"Changelog - v0.5.1","type":"extras"},{"doc":"Support :map which accepts the same :keys specification as keyword lists Normalize all error messages to include the key and expected value out of the box Do not nest options when rendered in Markdown and make sure multiline content is properly indented Handle \\r\\n style of line breaks in docs Automatically add types to generated docs Support lists of keyword lists in :list Add the :reference option type Add the :tuple option type","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Add support for all enumerables in {:in, choices} instead of just lists. You can now do things such as {:in, 1..10} . Deprecate the :rename_to schema option and emit a warning when used. Remove the {:one_of, choices} type which was deprecated in v0.3.3.","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"Add NimbleOptions.new!/1 to validate the schema once.","ref":"changelog.html#v0-3-7","title":"Changelog - v0.3.7","type":"extras"},{"doc":"Add :float type. Fix docs generation when custom key type has no keys.","ref":"changelog.html#v0-3-6","title":"Changelog - v0.3.6","type":"extras"},{"doc":"Add support for the {:list, subtype} type.","ref":"changelog.html#v0-3-5","title":"Changelog - v0.3.5","type":"extras"},{"doc":"Support nested schemas in the {:or, subtypes} type as {:or, [:string, keyword_list: [enabled: [type: :boolean]]]} . Improve validation of the return value of {:custom, module, function, args} functions. Support options in NimbleOptions.docs/2 . For now only the :nest_level option is supported.","ref":"changelog.html#v0-3-4","title":"Changelog - v0.3.4","type":"extras"},{"doc":"Add the {:or, subtypes} type. Deprecate the {:one_of, choices} and replace it with {:in, choices} . Using {:one_of, choices} emits a warning now.","ref":"changelog.html#v0-3-3","title":"Changelog - v0.3.3","type":"extras"},{"doc":"Fix a small bug with docs for nested schemas.","ref":"changelog.html#v0-3-2","title":"Changelog - v0.3.2","type":"extras"},{"doc":"Return :key and :value on %NimbleOptions.ValidationError{} to allow programmatic use of errors. Validate default values according to the specified type.","ref":"changelog.html#v0-3-1","title":"Changelog - v0.3.1","type":"extras"},{"doc":"Breaking change : return {:error, %NimbleOptions.ValidationError{}} tuples when there's a validation error in NimbleOptions.validate/2 instead of {:error, message} (with message being a string). You can use Exception.message/1 to turn the NimbleOptions.ValidationError struct into a string. Add the :pid type.","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Add NimbleOptions.validate!/2 .","ref":"changelog.html#v0-2-1","title":"Changelog - v0.2.1","type":"extras"},{"doc":"Change the behavior of NimbleOptions.docs/1 to accept a normal schema and produce documentation for that. Add support for doc: false as a schema option to hide an option or an option and its subsection.","ref":"changelog.html#v0-2-0","title":"Changelog - v0.2.0","type":"extras"},{"doc":"First release.","ref":"changelog.html#v0-1-0-2020-04-07","title":"Changelog - v0.1.0 (2020-04-07)","type":"extras"}]